@@ -19,7 +19,6 @@
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
 
-#include <glib.h>
 #include <stdbool.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -34,13 +33,12 @@
 #include "ar_dbus.h"
 #include "cmds.h"
 #include "common.h"
-#include "parse.h"
 #include "utils.h"
+#include "var.h"
 
-extern int macroCmd(char *descr, char *exec, cmdParams* params);  // from cmds.c
+extern int macroCmdCallback(char *descr, char *exec, cmdParams* params);  // from cmds.c
 
 extern char tmp[MAXMAXLEN];
-extern GMainLoop *loop;
 
 typedef struct Dp {
     char        *id;
@@ -84,6 +82,7 @@ int getParamType(char* tc)
 }
 
 #ifdef USE_DBUS
+
 DBusGConnection *connection = NULL;
 type_dp         *proxies    = NULL;
 type_cb         *callbacks  = NULL;
@@ -97,7 +96,7 @@ void intCb (DBusGProxy *proxy, int data, gpointer user_data)
     strcpy(p.index,"-1");
     sprintf(p.value,"%d",data);
 
-    macroCmd((char*) user_data, NULL, &p);
+    macroCmdCallback((char*) user_data, NULL, &p);
 }
 
 void uintCb (DBusGProxy *proxy, uint data, gpointer user_data)
@@ -109,7 +108,7 @@ void uintCb (DBusGProxy *proxy, uint data, gpointer user_data)
     strcpy(p.index,"-1");
     sprintf(p.value,"%u",data);
 
-    macroCmd((char*) user_data, NULL, &p);
+    macroCmdCallback((char*) user_data, NULL, &p);
 }
 
 void doubleCb (DBusGProxy *proxy, double data, gpointer user_data)
@@ -121,7 +120,7 @@ void doubleCb (DBusGProxy *proxy, double data, gpointer user_data)
     strcpy(p.index,"-1");
     sprintf(p.value,"%f",data);
 
-    macroCmd((char*) user_data, NULL, &p);
+    macroCmdCallback((char*) user_data, NULL, &p);
 }
 
 void boolCb (DBusGProxy *proxy, gboolean data, gpointer user_data)
@@ -133,7 +132,7 @@ void boolCb (DBusGProxy *proxy, gboolean data, gpointer user_data)
     strcpy(p.index,"-1");
     sprintf(p.value,"%s",(data ? "true" : "false"));
 
-    macroCmd((char*) user_data, NULL, &p);
+    macroCmdCallback((char*) user_data, NULL, &p);
 }
 
 void stringCb (DBusGProxy *proxy, const char *message, gpointer user_data)
@@ -145,7 +144,7 @@ void stringCb (DBusGProxy *proxy, const char *message, gpointer user_data)
     strcpy(p.index,"-1");
     strncpy(p.value,message,MAXARGLEN);
 
-    macroCmd((char*) user_data, NULL, &p);
+    macroCmdCallback((char*) user_data, NULL, &p);
 }
 
 static type_dp* findProxy(char *id, type_dp** prev)
@@ -275,12 +274,9 @@ static void freeCbs()
     }
 }
 
-#endif
-
 int dbusInit()
 {
     //logger(L_DBG, "dbusInit");
-#ifdef USE_DBUS
     GError *error = NULL;
 
     g_type_init ();
@@ -294,18 +290,13 @@ int dbusInit()
         return EXIT_NOK;
     }
     return EXIT_OK;
-#else
-    return EXIT_NOK;
-#endif
 }
 
 void dbusFinish()
 {
-#ifdef USE_DBUS
     freeProxies();
     freeCbs();
     dbus_g_connection_unref(connection);
-#endif
 }
 
 //
@@ -315,9 +306,10 @@ void dbusFinish()
 // get,_id_,_method_,int|double|bool|string,_variable_name_
 // signal,_id_,_method_,int|double|bool|string,_callback_id_
 //
-int dbusCommand(char* cmdString)
+int dbusCommand(int subtype, const char *descr, const char* cmdString, cmdParams* params)
 {
-#ifdef USE_DBUS
+    logger(L_INF, "Command: Dbus");
+    
     GError   *error  = NULL;
     gboolean retCode = TRUE;
 
@@ -325,13 +317,13 @@ int dbusCommand(char* cmdString)
         logger(L_ERR, "[EX]: dbusCommand(): no connection");
         return EXIT_NOK;
     }
-    if (cmdString == NULL) {
+    if (descr == NULL) {
         logger(L_ERR, "[EX]: dbusCommand(): no command");
         return EXIT_NOK;
     }
-    DEBUG2("[EX]: dbusCommand(): %s", cmdString)
+    DEBUG2("[EX]: dbusCommand(): %s", descr)
 
-    char * cmd = strdup(cmdString);
+    char * cmd = strdup(descr);
 
     char* token = strtok(cmd,",");
     while (token) {
@@ -347,9 +339,10 @@ int dbusCommand(char* cmdString)
             if (id != NULL && serv != NULL && path != NULL && intf != NULL) {
 
                 DBusGProxy  *proxy = dbus_g_proxy_new_for_name (connection,
-                                     serv, 		// ex.: "org.freedesktop.DBus",
+                                     serv, 	// ex.: "org.freedesktop.DBus",
                                      path,   	// ex.: "/org/freedesktop/DBus",
-                                     intf); 		// ex.: "org.freedesktop.DBus");
+                                     intf); 	// ex.: "org.freedesktop.DBus");
+				     
                 addProxy(id,serv,path,intf,proxy);	// proxy can be NULL here
                 if (proxy == NULL)  {
                     logger(L_DBG, "[EX]: dbusCommand(): get NULL proxy");
@@ -632,7 +625,27 @@ int dbusCommand(char* cmdString)
     }
 
     free(cmd);
-#endif
 
     return EXIT_OK;
 }
+
+#else
+
+// Just simple stubs to make compilation possible
+
+int dbusInit()
+{
+    return EXIT_NOK;
+}
+
+void dbusFinish()
+{
+}
+
+int dbusCommand(int subtype, const char *descr, const char* cmdString, cmdParams* params)
+{    
+    return EXIT_OK;
+}
+
+
+#endif
