@@ -22,44 +22,24 @@
 #ifndef _BTIO_H
 #define _BTIO_H
 
-int  rfcommConnect(const char* btAddress, int channel);
-void rfcommClose(void);
-
-int  openSerialPort (char* port);
-int  closeSerialPort(int final);
+#include "dispatcher.h"
+#include "peer.h"
 
 
-int  bt_read(char* buf, int len);
-int  bt_readchar(char* c, int timeout);
-int  bt_write(char* buf, int len);
-int  bt_put_command(const char* command,
+//int  bt_read(int portfd, char* buf, int len);
+int  bt_readchar(int portfd, char* c, int timeout);
+int  bt_put_command(int portfd, 
+                    const char* command,
                     char* answer,
                     int   max,
                     int   timeout,
                     char* expect);
 
-int  bt_getfd(void);
-
-char * getBtAddress(void);
-void freeBtAddress (void);
-
-int closeSocketPort(int type, int final);
-int openSocketPort(int type, int port,char *path);
-
-void openStdin();
-void closeStdin();
-int readStdin(char* buf, int max);
-
-int unix_open_port (char* port);
-int unix_close_port();
+int uxFD (ConnectInfo* connInfo);
+int  openUnixSocket (ConnectInfo* connInfo);
+void closeUnixSocket(ConnectInfo* connInfo, int final);
 //int unix_read_port(char* buf, int l);
 
-int listenAndAcceptSocketConn(int type);
-int writeSocketConn(const char* command, int count);
-int writeBytesSocketConn(char* command);
 int readSocketConn(char* buffer, int len);
-int sendKeepAlive(void);
-
-void sendIViewerHeartbeat(void);
 
 #endif
