@@ -23,8 +23,8 @@
 #define _COMMON_H_
 
 #define EXIT_EXACT  -3
-#define EXIT_NOK    -1
-#define EXIT_OK      0
+#define EXIT_NOK     0
+#define EXIT_OK      1
 #define EXIT_DISCON  2
 #define EXIT_ABORT   3
 #define EXIT_STOP    4
@@ -40,35 +40,43 @@
 //#define DEFAULT_DEVICE  "/dev/rfcomm0"	seems JSR-82 phones is more usable now
 #define DEFAULT_DEVICE  "bluetooth:19"
 
-#define AT_DEVICE   	"rfcomm"
+#define RFCOMM_DEVICE   "rfcomm"
+#define AT_DEVICE   	"at:"
 #define INET_SOCKET   	"socket:"
+#define PEER_TCP   	"tcp:"
 #define BT_SOCKET   	"bluetooth"
-#define BT_SPP   	"btspp" 	// not yet used
-#define L2CAP_SOCKET   	"l2cap"
+#define BT_SPP   	"btspp" 	
+#define L2CAP_SOCKET   	"l2cap"  	// not yet used
 #define UNIX_SOCKET   	"local:"
 #define ILIRC_SOCKET   	"ilirc:"
 #define STDIN_STREAM   	"stdin"
 #define WEB_SOCKET   	"web:"
+#define CMXML_SOCKET   	"cmxml:"
 
 #define TOFILE   	"/anyremote.result"
 #define CMDFILE   	"/anyremote.command"
 
+#define KEEPALIVE_TIMER "_PING_TIMER_"
+
 #define PROTO_AT        1
 #define PROTO_ANYREMOTE 2
 #define PROTO_BEMUSED   3
 #define PROTO_IVIEWER   4
 
-#define CLIENT_RFCOMM   1	// AT protocol over bluetooth
-#define CLIENT_AT       2	// AT protocol over IR or cable
-#define SERVER_TCP      3	// anyRemote protocol over TCP/IP
-#define SERVER_BT       4	// anyRemote protocol over btspp
-#define CLIENT_ILIRC    5	// inputLirc protocol
-#define CLIENT_NOAT     6	// anyRemote protocol over IR (or any AF_UNIX socket) - as client
-#define SERVER_STDIN    7	// anyRemote protocol, read command from stdin
-#define SERVER_WEB      8	// anyRemote protocol, built-in web server
-
-#define SERVER_L2CAP    101	// TODO: anyRemote protocol over l2cap
-#define SERVER_UX       102     // NOT USED: anyRemote protocol over IR (or any AF_UNIX socket) - as server
+enum ProtocolMode {
+    CLIENT_RFCOMM = 0,	 // AT protocol over bluetooth
+    CLIENT_AT,	         // AT protocol over IR or cable
+    SERVER_TCP,	         // anyRemote protocol over TCP/IP
+    SERVER_BT,	         // anyRemote protocol over btspp
+    CLIENT_ILIRC,	 // inputLirc protocol
+    CLIENT_NOAT,	 // anyRemote protocol over IR (or any AF_UNIX socket) - as client
+    SERVER_STDIN,	 // anyRemote protocol, read command from stdin
+    SERVER_WEB,	         // anyRemote protocol, built-in web server
+    SERVER_CMXML,	 // anyRemote protocol, built-in XML server
+    SERVER_L2CAP,	 // TODO: anyRemote protocol over l2cap
+    SERVER_UX ,  	 // NOT USED: anyRemote protocol over IR (or any AF_UNIX socket) - as server
+    SERVER_MAX   	 
+};
 
 #define DEFAULT_BT_CHANNEL 19
 #define BT_ADDR_LEN        17   // 00:12:EF:32:21:1A
@@ -78,9 +86,27 @@
 #define SOCK_CLOEXEC 0
 #endif
 
+#define VAR_AUTOCONN   	  "AutoConnect"
+#define VAR_AUTOREPEAT	  "AutoRepeat"
+#define VAR_CHARSET    	  "CharSet"
+#define VAR_TO_ENCODING   "ToEncoding"
+#define VAR_FROM_ENCODING "FromEncoding"
+#define VAR_LOGGING       "Logging"
+#define VAR_IVIEWER       "IViewer"
+#define VAR_BEMUSED       "Bemused"
+#define VAR_WAITEXEC      "WaitSeconds"
+#define VAR_ALLOWED_ONLY  "AllowedOnly"
+
+#define CMD_STR_DISCONNECT "Set(disconnect);"
+
 // TODO: Support L2CAP under FreeBSD
 //#ifdef USE_BLUEZ
 //#define USE_L2CAP 1
 //#endif
 
+enum RC_Code {
+    RC_NOK = 0,
+    RC_OK,
+};
+
 #endif
