@@ -26,67 +26,87 @@
 #include <termios.h>
 
 #include "parse.h"
+#include "var.h"
+#include "alarm.h"
 #include "utils.h"
+#include "mode.h"
 
 #define EMPTY_STR   	"EMPTY"
 
 extern char tmp[MAXMAXLEN];
 
-extern type_key* findExact(mode *mode, const char *key);
-
-extern mode       *modes;
-extern mode       *currentMode;
-extern mode       *internalMode;
-extern mode 	  *defMode;
 extern CONF       conf;
 
-type_alarm *alarms = NULL;
-int initDone = 0;
+int initDone  = 0;
+
+boolean_t doLog       = BOOL_NO;
+boolean_t doDebug     = BOOL_NO;
+boolean_t isIViewer   = BOOL_NO;
+boolean_t isBemused   = BOOL_NO;
+int       waitTimeout = -1;
+
+static char * _btAddress = NULL;
 
 /* ----------------- Interface functions ------------------------- */
 
-// caller function must free data
-static char* dupVarValue(const char *name, int size)
+boolean_t boolValue(const char* value)
 {
-    varData* v = searchVar(name, size);
-    if (v != NULL && v->value != NULL && v->value[0] != '\0') {
-        char *d = (char*) calloc(1,v->size+1);
-        strncpy(d,v->value,v->size);
-        d[v->size] = '\0';
-        return d;
+    if (!value) return BOOL_NO;
+    return (!value || strncmp(value,"false",5) == 0 || strncmp(value,"no",5) == 0 ? BOOL_NO : BOOL_YES);
+}
+
+void setLog(const char* value) 
+{
+    doLog = boolValue(value);
+    
+    doDebug = 0;
+    if (value && strncmp(value,"debug",5) == 0) {
+        doDebug = 1;
     }
-    return NULL;
 }
 
 int getLog()
 {
-    const char* v = getVarValue("Logging");
-    if (v != NULL && strncmp(v,"false",5) != 0) {
-        return 1;
-    }
-    return 0;
+    return doLog;
+}
+
+void setBemused(const char* value) 
+{
+    isBemused = boolValue(value);
+}
+
+int getBemused()
+{
+    return isBemused;
+}
+
+void setIViewer(const char* value) 
+{
+    isIViewer = boolValue(value);
+}
+
+int getIViewer()
+{
+    return isIViewer;
 }
 
 int getDebug()
 {
-    const char* v = getVarValue("Logging");
-    if (v != NULL && strncmp(v,"debug",5) == 0) {
-        return 1;
-    }
-    return 0;
+    return doDebug;
 }
 
 char* getDevice()
 {
-    return dupVarValue("Device", 6);
+    return dupVarValue("Device");
 }
 
 int getBaudrate()
 {
-    const char* value = getVarValue("Baudrate");
+    char* value = dupVarValue("Baudrate");
     if (value != NULL) {
 
         int rate = atoi(value);
+	free(value);
 
         switch (rate) {
         case 300:
@@ -123,36 +143,47 @@ int getBaudrate()
 
 int getRetrySecs()
 {
-    const char* value = getVarValue("RetrySeconds");
+    char* value = dupVarValue("RetrySeconds");
     if (value != NULL) {
-        return atoi(value);
+        int v = atoi(value);
+	free(value);
+	return v;
     }
     return 10;
 }
 
+int getWaitTime()
+{
+    return waitTimeout;
+}
+
+void setWaitTime(const char* value) 
+{
+    waitTimeout = -1;
+    
+    if (value != NULL) {
+        waitTimeout = atoi(value);
+	if (waitTimeout < 0) {
+	    waitTimeout = -1; 
+	}
+    }
+}
+
 int autoConnect()
 {
-    const char* v = getVarValue(AUTOCONN_STR);
-    if (v != NULL && strncmp(v,"true",4) == 0) {
+    int sz = 0;
+    const char* v = getVarValue(VAR_AUTOCONN, &sz);
+    if (v != NULL && sz > 3 && strncmp(v,"true",4) == 0) {
         return 1;
     }
     return 0;
 }
 
-int getIViewer()
-{
-    const char* v = getVarValue("IViewer");
-    if (v != NULL && strncmp(v,"true",4) == 0) {
-        return 1;
-    }
-    return 0;
-}
-
-
 int getAutoRepeat()
 {
-    const char* v = getVarValue(AUTOREPEAT_STR);
-    if (v != NULL && strncmp(v,"true",4) == 0) {
+    int sz = 0;
+    const char* v = getVarValue(VAR_AUTOREPEAT, &sz);
+    if (v != NULL && sz > 3 && strncmp(v,"true",4) == 0) {
         return 1;
     }
     return 0;
@@ -172,8 +203,9 @@ type_key* repeatNow()
 
 int getUseScreen()
 {
-    const char* v = getVarValue("TwoWayComm");
-    if (v != NULL && strncmp(v,"true",4) == 0) {
+    int sz = 0;
+    const char* v = getVarValue("TwoWayComm", &sz);
+    if (v != NULL && sz > 3 && strncmp(v,"true",4) == 0) {
         return 1;
     }
     return 0;
@@ -181,23 +213,23 @@ int getUseScreen()
 
 char* getCharset()
 {
-    return dupVarValue(CHARSET_STR, 7);
+    return dupVarValue(VAR_CHARSET);
 }
 
 char* getToMainMenu()
 {
-    return dupVarValue("ToMainMenu", 10);
+    return dupVarValue("ToMainMenu");
 }
 
 char* getServiceName()
 {
-    return dupVarValue("ServiceName", 11);
+    return dupVarValue("ServiceName");
 }
 
 char* getAT_CMER(int what)
 {
     if (what == CMER_ON) {
-        char *cmer = dupVarValue("CmerOn", 6);
+        char *cmer = dupVarValue("CmerOn");
 
         if  (cmer == NULL) {
             if (conf.model == MODEL_MOTOROLA) {
@@ -217,7 +249,7 @@ char* getAT_CMER(int what)
     }
 
     if (what == CMER_OFF) {
-        char *cmer = dupVarValue("CmerOff", 7);
+        char *cmer = dupVarValue("CmerOff");
 
         if  (cmer == NULL) {
             if (conf.model == MODEL_MOTOROLA) {
@@ -243,20 +275,9 @@ int getFrontEnd()
     return conf.frontEnd;
 }
 
-int getHttp()
-{
-    return conf.http;
-}
-
-int usePassword()
-{
-    return conf.pass;
-}
-
-
 char* getCfgDir(void)
 {
-    return dupVarValue("CfgDir", 6);
+    return dupVarValue("CfgDir");
 }
 
 char* getConfigDir(void)
@@ -264,11 +285,6 @@ char* getConfigDir(void)
     return conf.cfgDir;
 }
 
-char *getMode (void)
-{
-    return (currentMode == NULL ? NULL : currentMode->name);
-}
-
 void setModel (char *answer)
 {
     if (answer == NULL) {
@@ -291,56 +307,49 @@ int getModel (void)
     return conf.model;
 }
 
+void  setBtAddress (char* a)
+{
+    _btAddress = a;
+}
+
+char * getBtAddress()
+{
+    return _btAddress;
+}
+
+void freeBtAddress()
+{
+    if (_btAddress) {
+        free(_btAddress);
+    }
+    _btAddress = NULL;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////////////////////////
 
-void addAlarm(char *file, char *macro)
+type_key* findExact(mode *mode, const char *key)
 {
-    sprintf(tmp,"addAlarm() >%s< >%s<\n", file,macro);
-    logger(L_DBG,tmp);
+    //logger(L_DBG, "findExact()");
 
-    type_alarm * v = (type_alarm *) calloc(sizeof(type_alarm),1);
-    v->file  = strdup(file);
-    v->macro = strdup(macro);
-
-    // Insert in head
-    if (alarms == NULL) { 		// Insert first
-        v->next = NULL;
-    } else {
-        v->next = alarms;
-    }
-    alarms = v;
-}
-
-static type_alarm *tk = NULL;
-
-char* getFirstAlarm(char** macro)
-{
-    if (alarms == NULL) {
-        //logger(L_DBG,"no any alarm found");
+    if (mode == NULL || key == NULL) {
+        logger(L_DBG, "findExact() input is empty ?");
         return NULL;
     }
-    tk = alarms;
+    type_key* It = mode->keys;
 
-    *macro = tk->macro;
-    return tk->file;
-}
+    // Search exact command
+    while (It != NULL && It->key != NULL && strcmp(It->key,key) != 0) {
+        //sprintf(tmp,"findExact search >%s< compare to >%s<", It->key, key);
+        //logger(L_DBG,tmp);
 
-char* getNextAlarm(char** macro)
-{
-    if (tk != NULL) {
-        tk = tk->next;
+        It = (type_key*) It->next;
+        //logger(L_DBG, "findExact() next loop");
     }
-
-    if (tk == NULL) {
-        return NULL;
-    }
-
-    *macro = tk->macro;
-    return tk->file;
+    //logger(L_DBG, "findExact() exiting");
+    return It;
 }
 
-/////////////////////////////////////////////////////////////////////////////////////////////////
-
 static type_key* findStartingWith(mode *mode, const char *key)
 {
     if (mode == NULL) {
@@ -350,6 +359,8 @@ static type_key* findStartingWith(mode *mode, const char *key)
 
     while (It && It->key != NULL) {
         char *start = strstr(It->key,key);
+	
+	//DEBUG2("findStartingWith() %s %s %s",It->key,key,(start?"FOUND":"NOTFOUND"));
 
         if (start && start == It->key) {    // We got a part of multi-key or parametrized command
 
@@ -380,13 +391,13 @@ static type_key* findStartingWith(mode *mode, const char *key)
 static type_key* findItemInMode(mode *mode, const char *key, int *flag, cmdParams *params)
 {
     if (mode == NULL) {
-        logger(L_DBG,"findItemInMode: mode is null");
+        logger(L_DBG,"[EX]: findItemInMode: mode is null");
         return NULL;
     }
-    int canLog = (strcmp(mode->name,"_INTERNAL_") != 0);
+    int canLog = (strcmp(mode->name->str,"_INTERNAL_") != 0);
 
     if(canLog) {
-        sprintf(tmp,"findItemInMode >%s,%s<", mode->name, key);
+        sprintf(tmp,"[EX]: findItemInMode >%s,%s<", mode->name->str, key);
         logger(L_DBG,tmp);
     }
 
@@ -397,7 +408,7 @@ static type_key* findItemInMode(mode *mode, const char *key, int *flag, cmdParam
     if ((start != NULL && finish == NULL) ||  // Command was read partially?
             (start == NULL && finish != NULL)) {  // Command was incorectrly formed ?
         if(canLog) {
-            logger(L_DBG,"findItemInMode: Incorrectly formed parametrized command. One brace is absent");
+            logger(L_DBG,"[EX]: findItemInMode: Incorrectly formed parametrized command. One brace is absent");
         }
         return NULL;
     }
@@ -410,22 +421,26 @@ static type_key* findItemInMode(mode *mode, const char *key, int *flag, cmdParam
 
     type_key* It = findExact(mode,key);	// Search exact command
     if (It) {
-        //logger(L_DBG,"Found exact command");
-        *flag = FLAG_EXACT;
+        if(canLog) {
+            logger(L_DBG,"[EX]: findItemInMode: found exact command");
+        }
+	*flag = FLAG_EXACT;
         return It;
     }
 
     It = findStartingWith(mode,key);	// Search as part of multikey sequence
     if (It) {
-        //logger(L_DBG,"Found part of multi-key command");
+        if(canLog) {
+            logger(L_DBG,"[EX]: findItemInMode: found part of multi-key command");
+        }
         *flag = FLAG_MULTIKEY;
         return It;
     }
 
     if (params == NULL) {
-        if(canLog) {
-            logger(L_DBG,"findItemInMode: No parameters suspected. Item does not found.");
-        }
+        //if(canLog) {
+        //    logger(L_DBG,"[EX]: findItemInMode: No parameters suspected. Item does not found.");
+        //}
         return NULL;
     }
 
@@ -447,14 +462,14 @@ static type_key* findItemInMode(mode *mode, const char *key, int *flag, cmdParam
         const char *comma = strstr(start+1,",");
         if (comma == NULL || comma > finish) { // Parametrized command from Java Client should be in a form like List(1,String1)
             if(canLog) {
-                logger(L_DBG,"findItemInMode: Incorrectly formed parametrized command");
+                logger(L_DBG,"[EX]: findItemInMode: Incorrectly formed parametrized command");
             }
             return NULL;
         }
 
         if (comma-start > 6) { // 65535 = max value in Comman Fusion iViewer
             if(canLog) {
-                logger(L_ERR,"findItemInMode: Received incorrect index!");
+                logger(L_ERR,"[EX]: findItemInMode: Received incorrect index!");
             }
             return NULL;
         }
@@ -465,9 +480,20 @@ static type_key* findItemInMode(mode *mode, const char *key, int *flag, cmdParam
         *(value+(finish-comma-1)) = '\0';
 
         if(canLog) {
-            sprintf(tmp,"Parametrized command parsed as >%s< >%s< >%s< ", tag,index,value);
+            sprintf(tmp,"[EX]: Parametrized command parsed as >%s< >%s< >%s< ", tag,index,value);
             logger(L_DBG,tmp);
         }
+	
+	char * decodedVal = value;
+	int  needFree = 0;
+	
+	#ifdef USE_ICONV
+	if (needConvert() == 1) {
+	    decodedVal = convCharsetSimple(value, CNV_TO);
+	    needFree = 1;
+	    DEBUG2("[EX]: Decoded string >%s< ", (decodedVal ? decodedVal : "NULL"));
+	}
+	#endif
 
         // Try to search explicitly specified parametrized command (for ex List(1) or List(commandX))
         // By index
@@ -478,23 +504,37 @@ static type_key* findItemInMode(mode *mode, const char *key, int *flag, cmdParam
 
         if (It) {
             if(canLog) {
-                logger(L_DBG,"Found exact (explicitly specified by index) parametrized command");
+                logger(L_DBG,"[EX]: Found exact (explicitly specified by index) parametrized command");
             }
             *flag = FLAG_EXACT;
+	    
+	    #ifdef USE_ICONV
+	    if (needFree) {
+	        free(decodedVal);
+	    }
+	    #endif
+	    
             return It;
         }
 
         // Then by value
         *(tag+(start-key+1)) = '\0';
-        strcat(tag,value);
+        strcat(tag,(decodedVal ? decodedVal : ""));
         strcat(tag,")");
         It = findExact(mode,tag);
 
         if (It) {
             if(canLog) {
-                logger(L_DBG,"Found exact (explicitly specified by value) parametrized command");
+                logger(L_DBG,"[EX]: Found exact (explicitly specified by value) parametrized command");
             }
             *flag = FLAG_EXACT;
+	    
+	    #ifdef USE_ICONV
+	    if (needFree) {
+	        free(decodedVal);
+	    }
+	    #endif
+	    
             return It;
         }
 
@@ -506,30 +546,101 @@ static type_key* findItemInMode(mode *mode, const char *key, int *flag, cmdParam
         strcat(keyAndBrace,"($");
 
         if(canLog) {
-            DEBUG2("Search findStartingWith() >%s< ", keyAndBrace);
+            DEBUG2("[EX]: Search findStartingWith() >%s< ", keyAndBrace);
         }
         It = findStartingWith(mode,keyAndBrace);
         free(keyAndBrace);
         if (It) {
             if(canLog) {
-                DEBUG2("Found parametrized command >%s,%s< ", index,value);
+                DEBUG2("Found parametrized command >%s,%s< ", index,decodedVal);
             }
             strcpy(params->index, index);
-            strcpy(params->value, value);
+            strcpy(params->value, decodedVal);
 
             *flag = FLAG_PARAMETR;
+	    
+	    #ifdef USE_ICONV
+	    if (needFree) {
+	        free(decodedVal);
+	    }
+	    #endif
+	    
             return It;
         }
+	
+	#ifdef USE_ICONV
+	if (needFree) {
+	    free(decodedVal);
+	}
+	#endif
     }
-    if(canLog) {
-        logger(L_DBG,"findItemInMode: Item does not found");
-    }
+    //if(canLog) {
+    //    logger(L_DBG,"[EX]: findItemInMode: not found");
+    //}
     return NULL;
 }
 
+type_key* findItemInModeAndParents(mode* curMode, const char *key, int *flag, cmdParams *params)
+{
+    DEBUG2("[EX]: findItemInModeAndParents() search in mode %s (parents %s)", curMode->name->str, (curMode->parent ? curMode->parent->str : "no parents"));
+    type_key* tk = findItemInMode(curMode, key, flag, params);
+    
+    if (tk == NULL) {
+    	
+    	if (curMode != getDefaultMode() && curMode->parent) {
+    	
+	
+    	    string_t* parent = stringNew(curMode->parent->str);
+    	    char* ptrptr;
+    	    char* pmode  = strtok_r(parent->str,",", &ptrptr); 
+    	    
+    	    while (pmode) {
+    	    
+    		    // firstly strip spaces from start and tail
+
+    		    while (*pmode == ' ' || *pmode == '\t') {
+    		        pmode++;
+    		        if (*pmode == '\0') {
+    			        break;
+    		        }
+    		    }
+
+    		    if (*pmode != '\0') {
+
+    		        char *p2 = pmode + strlen(pmode) - 1;
+
+    		        while (*p2 == ' ' || *p2 == '\t') {
+    			        *p2 = '\0';
+    			        p2--;
+    			        if (pmode == p2) {
+    			            break;
+    			        }
+    		        }
+
+    		        if (*pmode != '\0') {
+    			        mode* tryMode = findMode(pmode);
+    			        if (tryMode) {
+    			            tk = findItemInModeAndParents(tryMode, key, flag, params);
+    			            if (tk) {
+    				            break;
+    			            }
+    			        } else {
+                            ERROR2("[EX]: findItemInModeAndParents() parent mode %s not found",pmode);
+                        }
+    		        }
+    		    }
+    		    pmode  = strtok_r(NULL,",",&ptrptr);
+    	    }
+    	    stringFree(parent,BOOL_YES);
+    	}
+    }
+    
+    return tk;
+}
+
 type_key* findItem(const char *keyIn, int *flag, cmdParams *params)
 {
-    //logger(L_DBG,"findItem()");
+    //logger(L_DBG,"[EX]: findItem()");
     type_key *tk = NULL;
 
     if (keyIn == NULL) {
@@ -540,42 +651,31 @@ type_key* findItem(const char *keyIn, int *flag, cmdParams *params)
     memset(key,0,MAXARGLEN);
 
     if (*keyIn == '\0') {
-        logger(L_INF,"Got empty key");
+        logger(L_INF,"[EX]: Got empty key");
         strcpy(key, EMPTY_STR);
     } else {
         strncpy(key, keyIn, MAXARGLEN - 1);
     }
 
-    if (currentMode != NULL) {
+    mode* curMode = getCurrentMode();
+    
+    if (curMode) {
 
-        mode* tryMode = currentMode;
+        tk = findItemInModeAndParents(curMode, key, flag, params);
 
-        while (tryMode) {
-            tk = findItemInMode(tryMode, key, flag, params);
-
-            if (tk == NULL) {
-                if (tryMode != defMode && strlen(tryMode->parent) > 0) {
-                    tryMode = findMode(tryMode->parent);
-                } else {
-                    break;
-                }
-            } else {
-                break;
-            }
-        }
-
-        if (tk == NULL && currentMode != defMode) {
-            logger(L_DBG,"findItem() search in default");
+        mode* defMode = getDefaultMode();
+        if (tk == NULL && curMode != defMode) {
+            logger(L_DBG,"[EX]: findItem() search in default");
             tk = findItemInMode(defMode, key, flag, params);
         }
     }
 
     //  Search in "internal" commands
     if (tk == NULL) {
-        logger(L_DBG,"findItem() internal mode");
-        tk = findItemInMode(internalMode, key, flag, params);
+        logger(L_DBG,"[EX]: findItem() internal mode");
+        tk = findItemInMode(getInternalMode(), key, flag, params);
         if (tk != NULL && *flag == FLAG_MULTIKEY) {
-            logger(L_DBG,"findItem() multikey in internal command ?");
+            logger(L_DBG,"[EX]: findItem() multikey in internal command ?");
         }
     }
 
@@ -587,123 +687,101 @@ type_key* findItem(const char *keyIn, int *flag, cmdParams *params)
     return tk;
 }
 
-cmdItem *getCommand(type_key* item)
+SingleList *getCommand(type_key* item)
 {
     if (!item) {
-        logger(L_DBG,"getCommand(): Empty type_key was got");
+        //logger(L_DBG,"[EX]: getCommand(): Empty type_key was got");
         return NULL;
     }
     if (item->key == NULL) {
-        logger(L_DBG,"Empty key was got");
+        //logger(L_DBG,"[EX]: Empty key was got");
         return NULL;
     }
 
-    sprintf(tmp,"getCommand for >%s<", item->key);
-    logger(L_DBG,tmp);
+    //sprintf(tmp,"[EX]: getCommand for >%s<", item->key);
+    //logger(L_DBG,tmp);
 
-    return item->cmd;
+    return item->commands;
 }
 
 //
 // Print cfg stuff
 //
-
-void printKeysInMode(type_key* It)
+static void printKey(const char* key, SingleList* commands)
 {
-    while (It && It->key != NULL) {
-        if (It->cmd) {
-            sprintf(tmp, "\t%s = ", It->key);
+    sprintf(tmp, "%s=\\\n\t", key);
 
-            cmdItem* item = It->cmd;
-            while (item) {
+    SingleList* list = commands;
+    while (list) {
 
-                strcat(tmp, id2Cmd(item->type));
+        cmdItem * item = (cmdItem *) list->data;
+        if (!item) continue;
 
-                if (item->tdata || item->descr || item->exec) {
-                    strcat(tmp, "(");
-                }
+        strcat(tmp, id2Cmd(item->type));
 
-                int pComma = 0;
-                if (item->descr) {
-                    //printf("descr  %s\n",item->descr);
-                    strcat(tmp, item->descr);
-                    pComma = 1;
-                }
-                if (item->tdata ) {
-                    char iBuff[16];
-                    sprintf(iBuff,",%d",item->tdata->timeout);
-                    strcat(tmp, iBuff);
-                    sprintf(iBuff,",%d",item->tdata->times);
-                    strcat(tmp, iBuff);
-                    strcat(tmp, ",");
-                }
-                strcat(tmp, "<|>");
-                if (item->exec) {
-                    if (pComma) {
-                        strcat(tmp, ",");
-                    }
-                    strcat(tmp, item->exec);
-                }
-
-                if (item->tdata || item->descr || item->exec) {
-                    strcat(tmp, ")");
-                }
-                strcat(tmp, ";");
-                item = item->next;
-
-                if (item) {
-                    strcat(tmp, "\\");
-                }
-                logger(L_CFG, tmp);
-                sprintf(tmp, "\t\t");
+        if (item->descr || item->exec) {
+            strcat(tmp, "(");
+        }
 
+        int pComma = 0;
+        if (item->descr) {
+            //printf("descr  %s\n",item->descr);
+            strcat(tmp, item->descr);
+            pComma = 1;
+        }
+        strcat(tmp, "<|>");
+        if (item->exec) {
+            if (pComma) {
+                strcat(tmp, ",");
             }
+            strcat(tmp, item->exec);
+        }
+
+        if (item->descr || item->exec) {
+            strcat(tmp, ")");
+        }
+        strcat(tmp, ";");
+
+	    list = listSingleNext(list);
+
+        if (item  && list) {
+            strcat(tmp, "\\");
+        }
+        logger(L_CFG, tmp);
+        sprintf(tmp, "\t");
+
+    }
+}
+
+void printKeys(type_key* key)
+{
+    while (key && key->key != NULL) {
+    
+        SingleList* commands = getCommand(key);
+	
+        if (commands) {
+	    printKey(key->key, commands);
         } else {
-            sprintf(tmp, "\t%s = no command", It->key);
+            sprintf(tmp, "\t%s = no command", key->key);
             logger(L_CFG, tmp);
         }
 
-        It = (type_key*)It->next;
+        key = (type_key*) key->next;
     }
-    return;
-}
-
-void printKeys()
-{
-    mode *mp = modes;
-    while (mp) {
-
-        if (strlen(mp->parent) == 0) {
-            sprintf(tmp, "%s %s", MODE_STR, mp->name);
-        } else {
-            sprintf(tmp, "%s %s : %s", MODE_STR, mp->name, mp->parent);
-        }
-        logger(L_CFG, tmp);
-
-        type_key* It = mp->keys;
-
-        printKeysInMode(It);
-
-        sprintf(tmp, "%s %s", MODE_END_STR, mp->name);
-        logger(L_CFG, tmp);
-
-        mp = (mode*)mp->next;
-    }
-
-    return;
 }
 
 void  setInitDone(void)
 {
     initDone = 1;
 }
+
 void printConf()
 {
     //printf("printConf ENTER\n");fflush(stdout);
     if (initDone == 0) return; // to avoid dumping several times on initialization
     
-    printKeys   ();
-    printVars   ();
+    printModes();
+    printVars ();
 
     return ;
 }
@@ -712,85 +790,33 @@ void printConf()
 // Clean-up stuff
 //
 
-void freeCmds(cmdItem* item)
+static void destroyCommandItem(void* ptr) 
 {
-    cmdItem* mi;
-    while (item) {
-        mi = item->next;
-        if (item->descr != NULL) {
-            free(item->descr);
-            item->descr = NULL;
-        }
-        if (item->exec != NULL) {
-            free(item->exec);
-            item->exec = NULL;
-        }
-        if (item->tdata != NULL) {
-            free(item->tdata);
-            item->tdata = NULL;
-        }
-
-        free(item);
-        item = mi;
+    cmdItem* item = (cmdItem *) ptr;
+    
+    if (item->descr != NULL) {
+    	free(item->descr);
+    	item->descr = NULL;
     }
-    return;
+    if (item->exec != NULL) {
+    	free(item->exec);
+    	item->exec = NULL;
+    }
+    free(item);
 }
 
-void freeCfgEx(mode* pmodes, type_alarm* palarms)
+void freeCmds(SingleList* commands)
 {
-    //logger(L_DBG, "freeCfgEx()");
-
-    // Keys
-    while(pmodes) {
-        mode	 *md = pmodes;
-        //printf("mode -> %s\n",md->name);
-        type_key *mk = md->keys;
-
-        while (mk) {
-            type_key* It = mk;
-            mk = (type_key*) mk->next;
-
-            freeCmds(It->cmd);
-
-            It->cmd = NULL;
-            free(It->key);
-            free(It);
-        }
-        pmodes = (mode*) pmodes->next;
-        free(md);
-    }
-
-    // Alarms
-    type_alarm *mk = palarms;
-    while (mk) {
-        type_alarm* It = mk;
-        mk = (type_alarm*) mk->next;
-        free(It->file);
-        free(It->macro);
-        free(It);
+    if (commands) {
+        listSingleFullFree(commands, destroyCommandItem);
     }
 }
 
 void freeCfg(void)
 {
-    freeCfgEx(modes, alarms);
-
-    // special handling for "internal" mode
-    if (internalMode != NULL) {
-        type_key *mki = internalMode->keys;
-        while (mki) {
-            type_key* It = mki;
-            mki = (type_key*) mki->next;
-
-            freeCmds(It->cmd);
-
-            It->cmd = NULL;
-            free(It->key);
-            free(It);
-        }
-        free(internalMode);
-        internalMode = NULL;
-    }
+    logger(L_DBG, "freeCfg()");
+    
+    freeModes(NULL); // delete all modes stuff
 
     free(conf.cfgDir);
 }
