@@ -22,37 +22,35 @@
 #ifndef _DISPATCHER_H_
 #define _DISPATCHER_H_
 
-#include <glib.h>
+#include "str.h"
 
-#define DM_SET		1
-#define DM_SETFILE	2
-#define DM_SENDB	3
-#define DM_SENDS	4
-#define DM_CKPD 	5
-#define DM_CMER 	6
+enum DispMsgType {
+    DM_SET = 0,
+    DM_SETFILE,
+    DM_SENDB,
+    DM_SENDS,
+    DM_CKPD,
+    DM_CMER,
+    DM_EVENT_INIT,
+    DM_EVENT_DISCONNECT,
+    DM_EVENT_EXIT,
+    DM_TYPE_MAX
+};
 
-#define DM_EVENT_INIT       98
-#define DM_EVENT_DISCONNECT 99
-#define DM_EVENT_EXIT	    100
 
 typedef struct {
-    int        type;
-    int        size;
-    void*      value;
-    char*      file;
+    int   type;
+    int   size;
+    void* value;
+    char* file;
 } dMessage;
 
-typedef struct {
-    //int      proto;
-    int      mode;
-    int      port; 
-    GString  *portStr;
-    gboolean connected;  
-} ConnectInfo;
-
-gpointer dispatcherRoutine	(gpointer thread);
-void  	 sendToDispatcher  	(dMessage *buf);
-void     parseCommand           (char* cmd);
-void     closePort              (int final);
+pointer_t dispatcherRoutine (pointer_t thread);
+void  	  sendToDispatcher  (dMessage *buf);
+void  	  freeDMessage      (void* ptr);
+void      parseCommand      (char* cmd);
+void      closePort         (int final);
+char*     readFromFile      (char *cmdTag, char* file, int* size);
+dMessage* getDFinalizer     (void);
 
 #endif
