@@ -24,8 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <glib.h>
-
+#include "lib_wrapper.h"
 #include "ar_dbus.h"
 #include "common.h"
 #include "utils.h"
@@ -34,21 +33,17 @@
 #include "executor.h"
 #include "dispatcher.h"
 #include "pr_frontend.h"
+#include "queue.h"
+#include "timer.h"
+#include "var.h"
+#include "peer.h"
 
 extern int   getClip 	(char *string); // from atsend.h
 
 extern char	tmp[MAXMAXLEN];
-extern GAsyncQueue *q2exec;
-extern GAsyncQueue *q2main;
 
 extern int  gotSignal;
-extern gboolean stillRun;
-
-extern int  useCallId;
-extern int  callOn;
-extern int  callTimer;
-extern char callerId[MAXLEN];
-extern ConnectInfo connectInfo;
+extern boolean_t stillRun;
 
 static int  handle_command(char* cmd);
 static int  handle_alarm  (char* cmd) ;
@@ -62,7 +57,6 @@ static void handleInit();
 static void handleConnect();
 static void handleDisconnect();
 static void handleExit();
-static void handleEndcall();
 
 static void cleanAutoRepeatFlag();
 
@@ -73,15 +67,27 @@ char modifierString [MAXARGLEN];
 
 int  flushConf    = 0;
 
+void freeEMessage(void* ptr)
+{
+    eMessage *em = (eMessage *) ptr;
+    if (em) {
+        free(em->value);
+        free(em);
+    }
+}
+
 void sendToExecutor(eMessage *buf)
 {
-    if (q2exec == NULL || buf == NULL) {
-        return;
-    }
+    queuePush(Q_EXEC, (void*) buf);
+}
 
-    g_async_queue_push(q2exec,buf);
+static eMessage* getEFinalizer()
+{
+    eMessage* em = (eMessage*) malloc(sizeof(eMessage));
+    em->type  = EM_AS_IS;
+    em->value = strdup("End();");
 
-    return;
+    return em;
 }
 
 void sendEventToExecutor(int event)
@@ -92,31 +98,20 @@ void sendEventToExecutor(int event)
     *((int *)em->value) = event;
 
     sendToExecutor(em);
-    
-    if (getHttp() || connectInfo.mode == SERVER_WEB) {
 
-        eMessage* em = (eMessage*) malloc(sizeof(eMessage));
-        em->type  = EM_AS_IS;
-        em->value = strdup("End();");
-        sendToExecutor(em);
+    if (needFinalizer() == EXIT_OK) {
+        sendToExecutor(getEFinalizer());
     }
 }
 
-gpointer executorRoutine(gpointer thread)
+static void initExecutor()
 {
-    if (q2exec == NULL) {
-        logger(L_INF,"[EX]: Do not start executor thread");
-        return NULL;
-    }
-
-    logger(L_INF,"[EX]: Start executor thread");
-    int exitFlag = 0;
-
     dbusInit();
     //logger(L_INF,"[EX]: DBUS initialized");
+
     handleInit();
     //logger(L_INF,"[EX]: init hooks done");
-	   
+
     setInitDone();
     printConf();
 
@@ -125,130 +120,148 @@ gpointer executorRoutine(gpointer thread)
     dm->value = (void*) NULL;
     dm->type  = DM_EVENT_INIT;
     dm->file  = NULL;
-    
+
     sendToDispatcher(dm);
+}
+
+// Should we auto-repeat command (timeout about 1/10 second) ?
+static void doAutoRepeat()
+{
+    // Should we auto-repeat command (timeout about 1/10 second) ?
+    if (repeatNow() && getAutoRepeat()) {
+        int isOdd = ((int)repeatTimer/2)*2;
+
+        if (repeatTimer > 100) {  // Check for infinite loop. Only 50 autorepeats per one press
+            cleanAutoRepeatFlag();
+            return;
+        } else if (repeatTimer > 0 && isOdd == repeatTimer) {
+            DEBUG2("[EX]: Auto repeat command ... (%d)", repeatTimer);
+            handleCmdByKey(repeatNow(),NULL);
+        }
+        repeatTimer++;
+    }
+}
+
+static void doTimers()
+{
+    //DEBUG2("[EX]: check timers(%d)", commandTimer);
+    // Verify commands executed by timer (timeout about 1 sec)
+    if (commandTimer == 50) {
+        //logger(L_DBG,"[EX]: Verify timer commands ...");
+        verifyTimerCfg(1) ;
+        commandTimer = 0;
+    } else {
+        commandTimer++;
+    }
+}
+
+static int handleKeyMsg(void * ptr)
+{
+    char * cmd = (char *) ptr;
+    sprintf(tmp, "[EX]: got key >%s<", (cmd ? cmd : "NULL"));
+    logger(L_DBG, tmp);
+    handle_command(cmd);
+    return 0;
+}
+
+static int handleStringMsg(void * ptr)
+{
+    char * msgIn = (char*) ptr;
+    sprintf(tmp, "[EX]: got string >%s<", msgIn);
+    logger(L_DBG, tmp);
+    execDynamically(msgIn);
+    return 0;
+}
+
+static int handleEventMsg(void * ptr)
+{
+    int* evt = (int*) ptr;
+
+    int exitFlag = 0;
+
+    if (*evt == ID_EVT_CONNECT) {
+        logger(L_DBG, "[EX]: got event EVT_CONNECT");
+        handleConnect();
+    } else if (*evt == ID_EVT_DISCONNECT) {
+        logger(L_DBG, "[EX]: got event EVT_DISCONNECT");
+        handleDisconnect();
+    //} else if (*evt == ID_EVT_INIT) {
+        //logger(L_DBG, "[EX]: got event EVT_INIT");
+        //  handleInit();
+    } else if (*evt == ID_EVT_EXIT) {
+        logger(L_DBG, "[EX]: got event EVT_EXIT");
+        handleExit();
+        exitFlag = 1;
+    }
+    return exitFlag;
+}
+
+static int handleAlarmMsg(void * ptr)
+{
+    char * cmd = (char *) ptr;
+    sprintf(tmp, "[EX]: got alarm >%s<", cmd);
+    logger(L_DBG, tmp);
+    handle_alarm(cmd);
+    return 0;
+}
+
+static int handleAsIsMsg(void * ptr)
+{
+    logger(L_DBG, "[EX]: got as is");
+    char * cmd = (char *) ptr;
+
+    dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
+    dm->type  = DM_SET;
+    dm->size  = strlen(cmd);
+    dm->value = (void*) strdup(cmd);
+    dm->file  = NULL;
+
+    sendToDispatcher(dm);
+    return 0;
+}
+
+static struct {
+    int id;
+    int (*hook)(void* p);
+} _msgHooks[] = {
+    {EM_KEY,    handleKeyMsg   },
+    {EM_STRING, handleStringMsg},
+    {EM_EVENT,  handleEventMsg },
+    {EM_ALARM,  handleAlarmMsg },
+    {EM_AS_IS,  handleAsIsMsg  }
+};
+
+pointer_t executorRoutine(pointer_t thread)
+{
+    if (queueExists(Q_EXEC) != RC_OK) {
+        logger(L_INF,"[EX]: Do not start executor thread");
+        return NULL;
+    }
+
+    logger(L_INF,"[EX]: Start executor thread");
+    initExecutor();
 
     while (stillRun) {
 
-        //logger(L_DBG, "[EX]: LOOP");
-
-        eMessage *em = (eMessage *) g_async_queue_try_pop(q2exec);
-
+        eMessage *em = (eMessage *) queuePop(Q_EXEC);
         if (em != NULL) {
-
             //logger(L_DBG, "[EX]: got event");
 
-            if (em->type == EM_KEY) {
+            int exitFlag = (_msgHooks[em->type].hook)(em->value);
 
-                char * cmd = (char *) em->value;
-
-                sprintf(tmp, "[EX]: got key >%s<", (cmd ? cmd : "NULL"));
-                logger(L_DBG, tmp);
-
-                handle_command(cmd);
-                free(em->value);
-
-            } else if (em->type == EM_ITEM) {
-
-                cmdItem *pg = (cmdItem*) em->value;
-
-                logger(L_DBG, "[EX]: got cmdItem");
-
-                processCommands(pg, NULL);
-
-            } else if (em->type == EM_STRING) {
-
-                char * msgIn = (char*) em->value;
-
-                sprintf(tmp, "[EX]: got string >%s<", msgIn);
-                logger(L_DBG, tmp);
-
-                execDynamically(msgIn);
-                free(em->value);
-
-            } else if (em->type == EM_EVENT) {
-
-                int* evt = (int*) em->value;
-
-                sprintf(tmp, "[EX]: got event >%d<", *evt);
-                logger(L_DBG, tmp);
-
-                if (*evt == ID_EVT_CONNECT) {
-                    handleConnect();
-                } else if (*evt == ID_EVT_DISCONNECT) {
-                    handleDisconnect();
-                    //} else if (*evt == ID_EVT_INIT) {
-                    //	handleInit();
-                } else if (*evt == ID_EVT_ENDCALL) {
-                    handleEndcall();
-                } else if (*evt == ID_EVT_EXIT) {
-                    handleExit();
-                    exitFlag = 1;
-                }
-                free(em->value);
-
-            } else if (em->type == EM_ALARM) {
-
-                char * cmd = (char *) em->value;
-
-                sprintf(tmp, "[EX]: got alarm >%s<", cmd);
-                logger(L_DBG, tmp);
-
-                handle_alarm(cmd);
-
-                free(em->value);
-
-            } else if (em->type == EM_AS_IS) {
-
-                logger(L_DBG, "[EX]: got as is");
-
-                dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-                dm->type  = DM_SET;
-                dm->size  = strlen((char*)em->value);
-                dm->value = (void*) strdup((char*)em->value);
-		dm->file  = NULL;
-		
-                sendToDispatcher(dm);
-
-                free(em->value);
-
-            }
-            free(em);
+            freeEMessage(em);
 
             if (exitFlag) {
-                return NULL;
+                break;
             }
         }
         //logger(L_DBG, "[EX]: command processed of empty input");
 
         // Should we auto-repeat command (timeout about 1/10 second) ?
-        if (repeatNow() && getAutoRepeat()) {
-            int isOdd = ((int)repeatTimer/2)*2;
+        doAutoRepeat();
 
-            if (repeatTimer > 100) {  // Check for infinite loop. Only 50 autorepeats per one press
-                cleanAutoRepeatFlag();
-            } else if (repeatTimer > 0 && isOdd == repeatTimer) {
-                DEBUG2("[EX]: Auto repeat command ... (%d)", repeatTimer);
-
-                handleCmdByKey(repeatNow(),NULL);
-
-                repeatTimer++;
-            } else {
-                //logger(L_DBG,"[EX]: Auto repeat command ...Skip");
-                repeatTimer++;
-            }
-        }
-
-        //DEBUG2("[EX]: check timers(%d)", commandTimer);
-        // Verify commands executed by timer (timeout about 1 sec)
-        if (commandTimer == 50) {
-            //logger(L_DBG,"[EX]: Verify timer commands ...");
-            verifyTimerCfg(1) ;
-            commandTimer = 0;
-
-        } else {
-            commandTimer++;
-        }
+        // Timers check (once per second)
+        doTimers();
 
         if (!stillRun) {
             logger(L_DBG,"[EX]: Break from loop ...");
@@ -259,6 +272,7 @@ gpointer executorRoutine(gpointer thread)
         usleep(20000); // loop timer (1/50 of second)
     }
 
+    DEBUG2("[EX]: thread stopped");
     return NULL;
 }
 
@@ -287,15 +301,8 @@ static void handleConnect()
     int ok2 = handleHook(ID_EVT_CONNECT);
 
     if (ok1 == EXIT_OK || ok2 == EXIT_OK) {
-        if (getHttp()) {
-
-            dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-            dm->type  = DM_SET;
-            dm->size  = 6;
-            dm->value = (void*) strdup("End();");
-	    dm->file  = NULL;
-	    
-            sendToDispatcher(dm);
+        if (needFinalizer() == EXIT_OK) {
+            sendToDispatcher(getDFinalizer());
         }
     }
 }
@@ -316,35 +323,35 @@ static void handleDisconnect()
     freeCachedData();
 }
 
-static void handleEndcall()
-{
-    int i = 0;
-    type_key *k = findItem(EVT_ENDCALL, &i, NULL);
-    if (k != NULL && i == FLAG_EXACT) {
-        logger(L_INF,"[EX]: EVT_ENDCALL found");
-        handleCmdByKey(k,NULL);
-    }
-}
-
 static void handleExit()
 {
-    printf("[EX]: handleExit\n");
+    //printf("[EX]: handleExit\n");
+
+    freeTimers(NULL);
+
+    //printf("[EX]: handleExit EVT_EXIT\n");
+
     int i;
     type_key *k = findItem(EVT_EXIT, &i, NULL);
     if (k && i == FLAG_EXACT) {
         logger(L_INF, "[EX]: Exec cmd on exit");
         handleCmdByKey(k,NULL);   // Since we exiting now we did not interested in return code
     }
-    printf("[EX]: handleExit handleHook\n");
+
+    //printf("[EX]: handleExit handleHook\n");
+
     handleHook(ID_EVT_EXIT);
 
-    printf("[EX]: handleExit free data\n");
+    //printf("[EX]: handleExit free data\n");
+
+    freeTimers(NULL);  // can we create timer in (Exit) ???
     freeVars();
     freeCachedData();
     freeCfg();
     freeRegexps();
     dbusFinish();
-    printf("[EX]: handleExit EXIT\n");
+
+    //printf("[EX]: handleExit EXIT\n");
 }
 
 static void cleanAutoRepeatFlag()
@@ -371,7 +378,7 @@ static int handle_alarm(char* key)
         handleCmdByKeyEx(k, NULL,0);
 
         if (flushConf == 1) {
-            logger(L_DBG, "Flush old configuration");
+            logger(L_DBG, "[EX]: Flush old configuration");
             flushConf = 0;
 
             flushOldConf();
@@ -428,7 +435,7 @@ static int handle_key_press(char* key,int isRepeatable)
             strcpy(modifierString, key);
         }
 
-        if(connectInfo.mode == CLIENT_RFCOMM || connectInfo.mode == CLIENT_AT) {
+        if (needAtMainMenuReturn()) {
             sendToMainMenu();
         }
     } else if (k && i == FLAG_PARAMETR) {
@@ -442,11 +449,11 @@ static int handle_key_press(char* key,int isRepeatable)
     } else {
         // User had pressed some key ...
         // Send ToMainMenu sequence of CKPD's to show main menu again
-        if (connectInfo.mode == CLIENT_RFCOMM || connectInfo.mode == CLIENT_AT) {
+        if (needAtMainMenuReturn()) {
             sendToMainMenu();
         }
 
-        logger(L_DBG, "No approprite key definition was found by findItem()");
+        logger(L_DBG, "No approprite key definition was found");
         if (modifierString[0] != '\0') {
             // Clean modifier string
             modifierString[0] = '\0';
@@ -544,47 +551,11 @@ static int handle_command(char* cmd)
         //logger(L_INF,"[EX]: Caller ID received. Skip it.");
         return EXIT_OK;
 
-    } else if (strstr(cmd, DEF_RING) != NULL) {	// Incoming call
-        logger(L_INF,"[EX]: Incoming call"); 	// This event sent periodically until used answered a call
-
-        if (callOn == 0) {
-            if (useCallId) {
-                //logger(L_DBG, "[EX]: Get CallerId on incoming call");
-                int ret = getClip(callerId);
-
-                INFO2("[EX]: CallerId is %s",callerId);
-
-                if(ret == EXIT_EXACT) { // Active call exists
-                    //	logger(L_DBG,"[EX]: Caller ID received");
-                    //} else {
-                    logger(L_DBG,"[EX]: Can't get Caller ID. Probably call was finished");
-                }
-            }
-            //
-            // May be we can get +CLIP: "+xxxxxxxxxxxx",145,,,,0< here ?
-            //
-
-            int i = 0;
-            type_key* k = findItem(EVT_INCALL, &i, NULL);
-
-            if (k && i == FLAG_EXACT) {
-                logger(L_DBG, "[EX]: Exec cmd on incoming call");
-                handleCmdByKey(k,NULL);
-            }
-
-            // If there is (EndCall) command,then set timer to determine when call ends
-            i = 0;
-            k = findItem(EVT_ENDCALL, &i, NULL);
-            if (k && i == FLAG_EXACT) {
-                logger(L_DBG, "[EX]: Set call timer");
-                callTimer = 0;
-                callOn    = 1;
-            }
-
-            return EXIT_OK;
-        } else {
-            logger(L_DBG,"[EX]: Skip it."); // This event sent periodically until used answered a call
-        }
+    } else if (strstr(cmd, DEF_RING) != NULL) {	    // Incoming call, this event sent periodically until used answered a call
+        
+	logger(L_INF,"[EX]: Incoming call notification. Skip it."); // Handled inside peer read function
+        return EXIT_OK;
+	
     } else if (isLircCommand(cmd)) {	    // LIRC tricks
         return EXIT_OK;
     } else if (isInpLircCommand(cmd)) {	    // inputLIRC tricks
@@ -593,7 +564,7 @@ static int handle_command(char* cmd)
         return EXIT_OK;
     } else if (isIViewerCommand(cmd)) {	    // CommandFusion iViewer protocol support
         return EXIT_OK;
-    } else { /*if (connectInfo.mode == SERVER_STDIN)*/     // Just direct input, try it
+    } else { // SERVER_STDIN ? Just direct input, try it
         //printf("DEBUG: Is it SERVER_STDIN ? %s\n", cmd);
 
         char *p = cmd;
@@ -625,7 +596,7 @@ void sendToMainMenu()
         dm->type  = DM_CKPD;
         dm->value = (void*) tmm;
         dm->size  = strlen((char*) dm->value);
-	dm->file  = NULL;
+        dm->file  = NULL;
 
         sendToDispatcher(dm);
     }
@@ -662,7 +633,7 @@ static int isBemusedCommand(char *cmdIn)
                 strncmp(cmd, "FFWD", 4) == 0 ||
                 strncmp(cmd, "GVOL", 4) == 0 ||
                 strncmp(cmd, "INF2", 4) == 0 ||
-                strncmp(cmd, "INF", 4) == 0 ||
+                strncmp(cmd, "INF",  3) == 0 ||
                 strncmp(cmd, "LIST", 4) == 0 ||
                 strncmp(cmd, "NEXT", 4) == 0 ||
                 strncmp(cmd, "PAUS", 4) == 0 ||
@@ -679,33 +650,37 @@ static int isBemusedCommand(char *cmdIn)
             handle = 1;
         }
 
-        if (strncmp(cmd, "REPT", 4) == 0 ||
+        if (strncmp(cmd, "VOLM", 4) == 0 ||
+                strncmp(cmd, "REPT", 4) == 0 ||
                 strncmp(cmd, "SHFL", 4) == 0) {
 
-            // Just "toggle" functionality
-            //unsigned char bc = ((unsigned char) *(cmd + 4));
-            //sprintf(paramValue, "%d",bc);
-
-            handle = 1;
-            shift  = 5;
-        }
-        if (strncmp(cmd, "VOLM", 4) == 0) {
             unsigned char bc;
 
             // this must be handled in dispatcher thread
-            if (strlen(cmd) == 4) {  	// read only VOLM without value to set
-                logger(L_ERR,"got VOLM without value");
-                return 1;		// do nothing, but treat it as Bemused command
+            if (strlen(cmd) == 4) {
+                if (strncmp(cmd, "VOLM", 4) == 0 && strlen(cmd) == 4) {  // read only VOLM without value to set
+                    ERROR2("[EX]: isBemusedCommand: %s without value", cmd);
+                    return 1;		// do nothing, but treat it as Bemused command
+                }
+
+                // It is possible for REPT/SHFL: toggle command
+                handle = 1;
+
+            } else {
+                bc = ((unsigned char) *(cmd + 4));
+
+                if (strncmp(cmd, "VOLM", 4) == 0) {
+                    // VOLM 0-255 -> 0%-100%
+                    sprintf(paramValue, "%d", ((int)bc * 100)/255);
+                    DEBUG2("[EX]: isBemusedCommand: VOLM parameter >%s<", paramValue);
+                } else {
+                    // 0/1
+                    sprintf(paramValue, "%d", ((int)bc == 0 ? 0 : 1));
+                    DEBUG2("[EX]: isBemusedCommand: %s parameter >%s<", cmd, paramValue);
+                }
+                handle = 1;
+                shift  = 5;
             }
-            bc = ((unsigned char) *(cmd + 4));
-
-            // VOLM 0-255 -> 0%-100%
-            sprintf(paramValue, "%d", ((int)bc * 100)/255);
-
-            DEBUG2("[EX]: isBemusedCommand: VOLM parameter >%s<", paramValue);
-
-            handle = 1;
-            shift  = 5;
         }
         if (strncmp(cmd, "SLCT", 4) == 0) {
 
@@ -871,6 +846,7 @@ static int isIViewerCommand(char *cmdIn)
     // m=[portrait|landscape]\3
     // p=value\3
     // sXX=value\3
+    // n=value
 
     DEBUG2("[EX]: isIViewerCommand: >%s<",cmdIn);
 
@@ -878,13 +854,14 @@ static int isIViewerCommand(char *cmdIn)
     strcpy(cmdTmp,cmdIn);
 
     if (!((cmdTmp[0] == 'a' && isdigit(cmdTmp[1])) ||
-            (cmdTmp[0] == 'd' && isdigit(cmdTmp[1])) ||
-            (cmdTmp[0] == 'l' && isdigit(cmdTmp[1])) ||
-            (cmdTmp[0] == 's' && isdigit(cmdTmp[1])) ||
-            (cmdTmp[0] == 'm' && cmdTmp[1] == '=')   ||
-            (cmdTmp[0] == 'p' && cmdTmp[1] == '=')   ||
-            strcmp(cmdTmp, "h=0") == 0 ||
-            strcmp(cmdTmp, "i=1") == 0
+          (cmdTmp[0] == 'd' && isdigit(cmdTmp[1])) ||
+          (cmdTmp[0] == 'l' && isdigit(cmdTmp[1])) ||
+          (cmdTmp[0] == 's' && isdigit(cmdTmp[1])) ||
+          (cmdTmp[0] == 'm' && cmdTmp[1] == '=')   ||
+          (cmdTmp[0] == 'p' && cmdTmp[1] == '=')   ||
+          (cmdTmp[0] == 'n' && cmdTmp[1] == '=')   ||
+          strcmp(cmdTmp, "h=0") == 0 ||
+          strcmp(cmdTmp, "i=1") == 0
          )) {
         free(cmdTmp);
         return 0;
@@ -905,8 +882,9 @@ static int isIViewerCommand(char *cmdIn)
 
     strcat(cmd,"(");
     if (cmdTmp[0] == 'm' ||
-            cmdTmp[0] == 'p' ||
-            cmdTmp[0] == 's') {
+        cmdTmp[0] == 'p' ||
+        cmdTmp[0] == 's' ||
+        cmdTmp[0] == 'n') {
 
         strcat(cmd,"-1,");
         if (token != NULL) {
