@@ -35,37 +35,28 @@
 #ifdef __FreeBSD__
 #include <signal.h>
 #endif
-#include <glib.h>
 
 #include "common.h"
+#include "lib_wrapper.h"
 #include "conf.h"
 #include "dispatcher.h"
 #include "executor.h"
 #include "pr_frontend.h"
 #include "utils.h"
+#include "queue.h"
+#include "thread.h"
+#include "loop.h"
+#include "alarm.h"
 
 extern void freeDisplay();	// from xemulate.h
 
-#define ALARM_CLEAN 	0
-#define ALARM_CHECK 	1
-
 extern CONF	conf;
-extern ConnectInfo connectInfo;
 extern int      state;
-extern gboolean dispIsJoinable;
+extern boolean_t dispIsJoinable;
 
 char tmp	    [MAXMAXLEN];
 
-GMainLoop	*loop 	 = NULL;
-GAsyncQueue     *q2disp  = NULL;
-GAsyncQueue     *q2fe 	 = NULL;
-GAsyncQueue     *q2exec	 = NULL;
-GAsyncQueue     *q2main	 = NULL;
-
-GThread 	*dispThread = NULL;
-GThread 	*execThread = NULL;
-
-gboolean        stillRun = TRUE;
+boolean_t        stillRun = BOOL_YES;
 
 /*#include <string>
 #include <vector>
@@ -77,9 +68,9 @@ std::map<int,std::string> m;*/
 void aboutToExit()
 {
     logger(L_DBG, "[ML]: aboutToExit");
-    g_main_loop_quit (loop);
+    loopStop();
 
-    stillRun = FALSE;
+    //stillRun = BOOL_NO; do it after exiting from other threads (executor needs to handle exit event)
 
     if (getFrontEnd() > 0) {
         disconnectFrontEnd();
@@ -87,29 +78,43 @@ void aboutToExit()
 
     logger(L_DBG, "[ML]: aboutToExit: join executor");
     sendEventToExecutor(ID_EVT_EXIT);
-    g_thread_join(execThread);
+
+    threadJoin(T_EXEC);
 
     if (dispIsJoinable) {	// dispatcher can be blocked in accept()
         logger(L_DBG, "[ML]: aboutToExit: join dispatcher");
-        
-	dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
+
+        dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
         dm->size  = 0;
         dm->value = (void*) NULL;
         dm->type  = DM_EVENT_EXIT;
-	dm->file  = NULL;
-	
+        dm->file  = NULL;
+
         sendToDispatcher(dm);
-        g_thread_join(dispThread);
+
+        threadJoin(T_DISP);
     }
 
+    stillRun = BOOL_NO;
+
     logger(L_DBG, "[ML]: aboutToExit: release main loop");
-    g_main_loop_unref (loop);
+    loopDestroy();
+
+    queueRemove(Q_FE,   freeFMessage);
+    queueRemove(Q_DISP, freeDMessage);
+    queueRemove(Q_EXEC, freeEMessage);
+    queueRemove(Q_MAIN, freeMMessage);
 
     freeDisplay();
-    freeTimers();
 
     logger(L_DBG, "[ML]: aboutToExit: EXIT");
 
+#ifdef USE_ICONV
+    closeConvertor();
+#endif
+
+    releaseLog();
+
     exit(0);
 }
 
@@ -135,74 +140,6 @@ void sig_kill(int i)
     aboutToExit();
 }
 
-// mode: 0 - clear alarms
-//       1 - if fired - process, then clear alarms
-void manageAlarms(int mode)
-{
-    char    *pm;
-    struct stat buf;
-
-    char    *pf = getFirstAlarm(&pm);
-    while (pf != NULL) {
-        //sprintf(tmp, "[ML]: check alarm %s", pf);
-        //logger(L_DBG, tmp);
-
-        // If file exists
-        int fs = stat(pf, &buf);
-        if (fs == 0) {
-            if (mode == ALARM_CLEAN) {
-                logger(L_DBG,"[ML]: Remove file");
-            } else {
-                INFO2("[ML]: Alarm %s fired", pf);
-
-                eMessage* em = (eMessage*) malloc(sizeof(eMessage));
-                em->type  = EM_ALARM;
-                em->value = strdup(pm);
-
-                sendToExecutor(em);
-            }
-            remove(pf);
-        }
-        pf = getNextAlarm(&pm);
-    }
-    return;
-}
-
-gboolean alarmTimeout(gpointer data)
-{
-    //logger(L_DBG,"[ML]: alarmTimeout");
-    manageAlarms(ALARM_CHECK);
-    return TRUE;
-}
-
-gboolean adminRoutine(gpointer data)
-{
-    int *mi = (int*) g_async_queue_try_pop(q2main);
-
-    if (mi != NULL) {
-
-        int m = *mi;
-        free(mi);
-
-        if (m == M_ABORT) {
-            logger(L_DBG,"[ML]: got exit request");
-            aboutToExit();	// exit here
-
-        } else if (m == M_DISCONNECT) {
-            logger(L_DBG,"[ML]: got disconnect request");
-
-            if (!autoConnect() &&
-                    (connectInfo.mode == CLIENT_RFCOMM ||
-                     connectInfo.mode == CLIENT_AT     ||
-                     connectInfo.mode == CLIENT_ILIRC)) {
-                logger(L_DBG,"[ML]: Exit because of disconnect");
-                aboutToExit();	// exit here
-            }
-        }
-    }
-    return TRUE;
-}
-
 static void createDataDir()
 {
     char dd[542];
@@ -216,10 +153,10 @@ static void createDataDir()
     } else {
         mkdir(dd, S_IRWXU);
         if(getuid()==0 && conf.uid) { // do not create as superuser
-           #ifdef __cplusplus
-           int dummy = 
-           #endif
-           chown(dd,conf.uid,conf.gid);
+#ifdef __cplusplus
+            int dummy =
+#endif
+                chown(dd,conf.uid,conf.gid);
         }
     }
 }
@@ -235,7 +172,7 @@ int main(int argc, char *argv[])
         printf("anyremote v%s\n", PACKAGE_VERSION);
         exit(0);
     }
-    
+
     createDataDir();
 
     int i;
@@ -244,7 +181,7 @@ int main(int argc, char *argv[])
             break;
         }
     }
-    
+
     init_cfg_dir(i<argc ? argv[i+1] : NULL);
 
     // needs to be after init_cfg_dir()
@@ -255,7 +192,7 @@ int main(int argc, char *argv[])
         freeCfg();
         exit(1);
     }
-    
+
     if (init_cfg(i<argc ? argv[i+1] : NULL) != EXIT_OK) {
         printf("Can not initialize configuration. Exiting.\n");
         printHelp();
@@ -263,8 +200,7 @@ int main(int argc, char *argv[])
         exit(1);
     }
 
-
-    g_thread_init(NULL);	// need to call it before initLog()
+    threadInit();		// need to call it before initLog()
 
     initLog();
     manageAlarms(ALARM_CLEAN);
@@ -279,28 +215,20 @@ int main(int argc, char *argv[])
 
     //logger(L_DBG,"g_main_loop_new");
 
-    loop = g_main_loop_new(NULL, FALSE);
 
-    q2main = g_async_queue_new();
+    queueNew(Q_MAIN);
 
-    q2disp = g_async_queue_new();
-    dispThread = g_thread_create (dispatcherRoutine, NULL, TRUE, NULL);
+    queueNew(Q_DISP);
+    threadNew(T_DISP,dispatcherRoutine,NULL,JOINABLE);
 
-    q2exec  = g_async_queue_new();
-    execThread = g_thread_create (executorRoutine, NULL, TRUE, NULL);
+    queueNew(Q_EXEC);
+    threadNew(T_EXEC,executorRoutine,NULL,JOINABLE);
 
     if (getFrontEnd() > 0) {
-        q2fe = g_async_queue_new();
-        g_timeout_add(200, frontendRoutine, NULL); 	// 1/5 of second
-    } else {
-        logger(L_DBG,"Do not setup front-end connection");
+        queueNew(Q_FE);
     }
 
-    g_timeout_add(100,   adminRoutine, NULL); 		// 1/10 of second
-    g_timeout_add(15000, alarmTimeout, NULL);
-
-    //logger(L_DBG,"g_main_loop_run");
-    g_main_loop_run(loop);
+    loopStart();
 
     return 0;
 }
