@@ -22,10 +22,12 @@
 #ifndef _PR_L2CAP_H
 #define _PR_L2CAP_H
 
-int openL2capPort(int port);
-int closeL2capPort(int final);
-int listenAndAcceptL2capConn();
-int writeL2capConn(char* command, int count);
-int writeBytesL2capConn(char* command);
+#include "peer.h"
+
+int  l2capFD    (ConnectInfo* c);
+int  l2capOpen  (ConnectInfo* c);
+void l2capClose (ConnectInfo* c,int final);
+int  l2capAccept(ConnectInfo* c);
+int  l2capWrite (ConnectInfo* c, dMessage* msg);
 
 #endif
