@@ -23,21 +23,7 @@
 #define _PR_WEB_H
 
 #include "dispatcher.h"
-
-enum arForm {
-    CF=1,
-    TX,
-    LI,
-    FM,
-    WM,
-    EF 
-};
-
-enum arCookieStatus {
-    NO_COOKIE=1,
-    NEED_SEND_COOKIE,
-    COOKIE_SENT
-};
+#include "peer.h"
 
 enum arRequestType {
     RQ_UNKNOWN=0,
@@ -47,19 +33,15 @@ enum arRequestType {
 
 typedef struct {
     int        button;
-    GString*   string;
+    string_t*  string;
 } wMessage;
 
-typedef struct {
-    GString*   icon;
-    GString*   string;
-} listItem;
+int  openWeb     (ConnectInfo* connInfo);
+void closeWebPort(ConnectInfo* connInfo, int final);
+int  checkWebPort();
+int  writeWebConn(const dMessage* dm, int mode);
+int  writeWeb    (ConnectInfo* peer, const dMessage* dm);
 
-int openWebPort (int port);
-void closeWebPort(int final);
-int checkWebPort();
-//int readWebPort (char *buf, int l);
-int writeWebConn(dMessage* dm);
-int writeWebConnStr(char* value);
+void freeWMessage(void *buf);
 
 #endif
