@@ -22,15 +22,12 @@
 #ifndef _UTILS_H
 #define _UTILS_H
 
-#include "parse.h"
 #include <sys/types.h>
 #include <pwd.h>
 
-#define LOGFILE 	"/anyremote.log"
+#include "parse.h"
 
-#define TIMER_RUN       1
-#define TIMER_PAUSE     2
-#define TIMER_CANCEL    3
+#define LOGFILE 	"/anyremote.log"
 
 enum LogType {
     L_CFG = 0,
@@ -50,44 +47,19 @@ enum LogType {
 #define M_DISCONNECT 	1
 #define M_DEV_DISCONN 	2
 
-typedef struct TC {
-    cmdItem *timer;
-
-    int	ticks;
-    int	times;
-
-    int     status;
-
-    struct TC  *next;
-} timerCmd;
-
-typedef struct VV {
-    char *name;
-    char *value;
-    int  size;
-    struct VV  *next;
-} varData;
+#define CNV_TO 	 1
+#define CNV_FROM 0
 
 void        printHelp       (void);
 void        printTime       (void);
 void        printVars       (void);
 void        initLog         (void);
+void        releaseLog      (void);
 void        logger          (int head, const char *str);
-//void        logger          (const char *head, const char *str);
 
-timerCmd*   createTimer     (cmdItem *ci);
-timerCmd*   findTimer       (char *id, timerCmd **prev);
-void        verifyTimerCfg  (int ticksInSec);
-void        cancelTimer     (char *id);
-void        freeTimers      (void);
+void        freeMMessage    (void *mm);
 
-int         setVarSimple    (const char *name, const char *val);
-int         setVar          (const char *name, const char *val, int size);
-varData*    searchVar       (const char *id, int len);
-void        freeVars        (void);
-const char*  getVarValue    (const char *name);
-
-int         isDataNew       (int what, char *data, int size);
+int         isDataNew       (int what, const char *data, int size);
 void        freeCachedData  (void);
 
 int         flushData       (void);
@@ -99,10 +71,20 @@ char*       getCmdFile      (void);
 
 int         getUidGid       (char *username, uid_t *uid, gid_t *gid);
 
-char* 	    getStoredPass   (void);
 void 	    addInternalVars (void);
 
 void  	    sendAbort       (void);
 void        sendDisconnect  (void);
 
+int         needConvert     (void);
+
+void        errnoDebug(const char* tag, int err);
+
+#ifdef USE_ICONV
+void        encodingHook     (void);
+void        closeConvertor   (void);
+char *      convCharset      (char *str, size_t size, int direction);
+char *      convCharsetSimple(char *str, int direction);
+#endif
+
 #endif
